searchNodes=[{"doc":"Semantic comparison and sorting for Elixir. Why Cmp ? The built-in comparison operators as well as functions like Enum.sort/2 or Enum.max/1 are based on Erlang's term ordering and suffer two issues, which require attention and might lead to unexpected behaviors or bugs: 1. Structural comparisons Built-ins use structural comparison over semantic comparison : ~D[2020-03-02] &gt; ~D[2019-06-06] false iex&gt; Enum . sort ( [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] ) [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] Semantic comparison is available but not straightforward: iex&gt; Date . compare ( ~D[2019-01-01] , ~D[2020-03-02] ) :lt iex&gt; Enum . sort ( [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] , Date ) [ ~D[2019-01-01] , ~D[2019-06-06] , ~D[2020-03-02] ] Cmp will do the right right out of the box: iex&gt; Cmp . gt? ( ~D[2020-03-02] , ~D[2019-06-06] ) true iex&gt; Cmp . sort ( [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] ) [ ~D[2019-01-01] , ~D[2019-06-06] , ~D[2020-03-02] ] 2. Weakly typed Built-ins will accept any set of operands and happily compare them: iex&gt; 2 &lt; &quot;1&quot; true iex&gt; 0 &lt; true true iex&gt; false &lt; nil true Cmp will only compare compatible elements or raise a Cmp.TypeError : iex&gt; Cmp . lte? ( 1 , 1.0 ) true iex&gt; Cmp . lte? ( 2 , &quot;1&quot; ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 2, right: &quot;1&quot; What's in the box Boolean comparisons: eq?/2 , lt?/2 , gt?/2 , lte?/2 , gte?/2 Equivalents of Kernel.min/2 / Kernel.max/2 : Cmp.min/2 , Cmp.max/2 Equivalents of Enum.min/1 / Enum.max/1 / Enum.sort/2 : Cmp.min/1 , Cmp.max/1 , Cmp.sort/2 compare/2 Supported types The Cmp.Comparable protocol is implemented for the following types: Integer Float Bitstring Date Time DateTime NaiveDateTime Version Tuple (see below) Decimal (if available) It isn't implemented for atoms by design, since atoms are not semantically an ordered type. It supports tuples of the same size and types: iex&gt; Cmp . max ( { 12 , ~D[2019-06-06] } , { 12 , ~D[2020-03-02] } ) { 12 , ~D[2020-03-02] } iex&gt; Cmp . max ( { 12 , &quot;Foo&quot; } , { 15 , nil } ) ** (Cmp.TypeError) Failed to compare incompatible types - left: &quot;Foo&quot;, right: nil iex&gt; Cmp . max ( { 12 , &quot;Foo&quot; } , { 15 } ) ** (Cmp.TypeError) Failed to compare incompatible types - left: {12, &quot;Foo&quot;}, right: {15} Decimal support can prevent nasty bugs too: iex&gt; max ( Decimal . new ( 2 ) , Decimal . from_float ( 1.0 ) ) # Decimal &lt; 1.0 &gt; iex&gt; Cmp . max ( Decimal . new ( 2 ) , Decimal . from_float ( 1.0 ) ) # Decimal &lt; 2 &gt; See the Cmp.Comparable documentation to implement the protocol for other existing or new structs. Design goals Fast and well-optimized - the overhead should be quite small over built-in equivalents. See the benchmarks/ folder for more details. No need to require macros, plain functions Easily extensible through the Cmp.Comparable protocol Robust and well-tested (both unit and property-based) Limitations Cmp comparators cannot be used in guards. Cmp does not support (or plan to support) comparisons between non-homogeneous types (e.g. Decimal and native numbers).","ref":"Cmp.html","title":"Cmp","type":"module"},{"doc":"Returns :gt if left is semantically greater than right , lt if left is less than right , and :eq if they are equal. Examples iex&gt; Cmp . compare ( 2 , 1 ) :gt iex&gt; Cmp . compare ( 1 , 1.0 ) :eq It will raise a Cmp.TypeError if trying to compare incompatible types iex&gt; Cmp . compare ( 1 , &quot;&quot; ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: &quot;&quot;","ref":"Cmp.html#compare/2","title":"Cmp.compare/2","type":"function"},{"doc":"Safe equivalent to ==/2 , which only works if both types are compatible and uses semantic comparison. Examples iex&gt; Cmp . eq? ( 2 , 1 ) false iex&gt; Cmp . eq? ( 1 , 1.0 ) true It will raise a Cmp.TypeError if trying to compare incompatible types iex&gt; Cmp . eq? ( 1 , &quot;&quot; ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: &quot;&quot;","ref":"Cmp.html#eq?/2","title":"Cmp.eq?/2","type":"function"},{"doc":"Safe equivalent to &gt;/2 , which only works if both types are compatible and uses semantic comparison. Examples iex&gt; Cmp . gt? ( 2 , 1 ) true iex&gt; Cmp . gt? ( 1 , 2 ) false iex&gt; Cmp . gt? ( 1 , 1.0 ) false It will raise a Cmp.TypeError if trying to compare incompatible types iex&gt; Cmp . gt? ( 1 , nil ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil Unlike &gt;/2 , it will perform a semantic comparison for structs and not a structural comparison : iex&gt; Cmp . gt? ( ~D[2020-03-02] , ~D[2019-06-06] ) true ~D[2020-03-02] &gt; ~D[2019-06-06] false","ref":"Cmp.html#gt?/2","title":"Cmp.gt?/2","type":"function"},{"doc":"Safe equivalent to &gt;=/2 , which only works if both types are compatible and uses semantic comparison. Examples iex&gt; Cmp . gte? ( 2 , 1 ) true iex&gt; Cmp . gte? ( 1 , 2 ) false iex&gt; Cmp . gte? ( 1 , 1.0 ) true It will raise a Cmp.TypeError if trying to compare incompatible types iex&gt; Cmp . gte? ( 1 , nil ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil Unlike &gt;=/2 , it will perform a semantic comparison for structs and not a structural comparison : iex&gt; Cmp . gte? ( ~D[2020-03-02] , ~D[2019-06-06] ) true ~D[2020-03-02] &gt;= ~D[2019-06-06] false","ref":"Cmp.html#gte?/2","title":"Cmp.gte?/2","type":"function"},{"doc":"Safe equivalent to &lt;/2 , which only works if both types are compatible and uses semantic comparison. Examples iex&gt; Cmp . lt? ( 1 , 2 ) true iex&gt; Cmp . lt? ( 2 , 1 ) false iex&gt; Cmp . lt? ( 1 , 1.0 ) false It will raise a Cmp.TypeError if trying to compare incompatible types iex&gt; Cmp . lt? ( 1 , nil ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil Unlike &lt;/2 , it will perform a semantic comparison for structs and not a structural comparison : iex&gt; Cmp . lt? ( ~D[2019-06-06] , ~D[2020-03-02] ) true ~D[2019-06-06] &lt; ~D[2020-03-02] false","ref":"Cmp.html#lt?/2","title":"Cmp.lt?/2","type":"function"},{"doc":"Safe equivalent to &lt;=/2 , which only works if both types are compatible and uses semantic comparison. Examples iex&gt; Cmp . lte? ( 1 , 2 ) true iex&gt; Cmp . lte? ( 2 , 1 ) false iex&gt; Cmp . lte? ( 1 , 1.0 ) true It will raise a Cmp.TypeError if trying to compare incompatible types iex&gt; Cmp . lte? ( 1 , nil ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil Unlike &lt;=/2 , it will perform a semantic comparison for structs and not a structural comparison : iex&gt; Cmp . lte? ( ~D[2019-06-06] , ~D[2020-03-02] ) true ~D[2019-06-06] &lt;= ~D[2020-03-02] false","ref":"Cmp.html#lte?/2","title":"Cmp.lte?/2","type":"function"},{"doc":"Safe equivalent of Enum.max/2 , returning the minimum of a non-empty enumerable of comparables. Examples iex&gt; Cmp . max ( [ 1 , 3 , 2 ] ) 3 Respects semantic comparison: iex&gt; Cmp . max ( [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] ) ~D[2020-03-02] Raises a Cmp.TypeError on non-uniform enumerables: iex&gt; Cmp . max ( [ 1 , nil , 2 ] ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil Raises an Enum.EmptyError on empty enumerables: iex&gt; Cmp . max ( [ ] ) ** (Enum.EmptyError) empty error","ref":"Cmp.html#max/1","title":"Cmp.max/1","type":"function"},{"doc":"Safe equivalent to max/2 , which only works if both types are compatible and uses semantic comparison. Examples iex&gt; Cmp . max ( 1 , 2 ) 2 iex&gt; Cmp . max ( 1 , 1.0 ) 1 It will raise a Cmp.TypeError if trying to compare incompatible types iex&gt; Cmp . max ( 1 , nil ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil Unlike max/2 , it will perform a semantic comparison for structs and not a structural comparison : iex&gt; Cmp . max ( ~D[2020-03-02] , ~D[2019-06-06] ) ~D[2020-03-02] max ( ~D[2020-03-02] , ~D[2019-06-06] ) ~D[2019-06-06]","ref":"Cmp.html#max/2","title":"Cmp.max/2","type":"function"},{"doc":"Safe equivalent of Enum.min/2 , returning the maximum of a non-empty enumerable of comparables. Examples iex&gt; Cmp . min ( [ 1 , 3 , 2 ] ) 1 Respects semantic comparison: iex&gt; Cmp . min ( [ ~D[2020-03-02] , ~D[2019-06-06] ] ) ~D[2019-06-06] Raises a Cmp.TypeError on non-uniform enumerables: iex&gt; Cmp . min ( [ 1 , nil , 2 ] ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil Raises an Enum.EmptyError on empty enumerables: iex&gt; Cmp . min ( [ ] ) ** (Enum.EmptyError) empty error","ref":"Cmp.html#min/1","title":"Cmp.min/1","type":"function"},{"doc":"Safe equivalent to min/2 , which only works if both types are compatible and uses semantic comparison. Examples iex&gt; Cmp . min ( 2 , 1 ) 1 iex&gt; Cmp . min ( 1 , 1.0 ) 1 It will raise a Cmp.TypeError if trying to compare incompatible types iex&gt; Cmp . min ( 1 , nil ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 1, right: nil Unlike min/2 , it will perform a semantic comparison for structs and not a structural comparison : iex&gt; Cmp . min ( ~D[2020-03-02] , ~D[2019-06-06] ) ~D[2019-06-06] min ( ~D[2020-03-02] , ~D[2019-06-06] ) ~D[2020-03-02]","ref":"Cmp.html#min/2","title":"Cmp.min/2","type":"function"},{"doc":"Safe equivalent of Enum.sort/2 . Examples iex&gt; Cmp . sort ( [ 3 , 1 , 2 ] ) [ 1 , 2 , 3 ] iex&gt; Cmp . sort ( [ 3 , 1 , 2 ] , :desc ) [ 3 , 2 , 1 ] Respects semantic comparison: iex&gt; Cmp . sort ( [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] ) [ ~D[2019-01-01] , ~D[2019-06-06] , ~D[2020-03-02] ] Raises a Cmp.TypeError on non-uniform enumerables: iex&gt; Cmp . sort ( [ ~D[2019-01-01] , nil , ~D[2020-03-02] ] ) ** (Cmp.TypeError) Failed to compare incompatible types - left: ~D[2019-01-01], right: nil","ref":"Cmp.html#sort/2","title":"Cmp.sort/2","type":"function"},{"doc":"A protocol to define how elements of a type can be compared. The simplest way to define it for a custom struct is to use @derive based on semantical comparison of fields, in the given order: defmodule MyStruct do @derive { Cmp.Comparable , using : [ :date , :id ] } defstruct [ :id , :date ] end Cmp . sort ( [ % MyStruct { date : ~D[2020-03-02] , id : 100 } , % MyStruct { date : ~D[2020-03-02] , id : 101 } , % MyStruct { date : ~D[2019-06-06] , id : 300 } ] ) [ % MyStruct { date : ~D[2019-06-06] , id : 300 } , % MyStruct { date : ~D[2020-03-02] , id : 100 } , % MyStruct { date : ~D[2020-03-02] , id : 101 } ] For existing structs that are already implementing a compare/2 function returning :eq | :lt | :gt , the protocol can simply be done by passing using: :compare . require Protocol Protocol . derive ( Cmp.Comparable , MyStruct , using : :compare ) This is already done for the following modules: Date Time DateTime NaiveDateTime Version Decimal (if available)","ref":"Cmp.Comparable.html","title":"Cmp.Comparable","type":"protocol"},{"doc":"Defines how to semantically compare two elements of a given type. This should return: :eq if left == right :lt if left &lt; right :gt if left &gt; right","ref":"Cmp.Comparable.html#compare/2","title":"Cmp.Comparable.compare/2","type":"function"},{"doc":"Type of an element implementing the Comparable protocol.","ref":"Cmp.Comparable.html#t:t/0","title":"Cmp.Comparable.t/0","type":"type"},{"doc":"","ref":"Cmp.Util.html","title":"Cmp.Util","type":"module"},{"doc":"","ref":"Cmp.Util.html#comparable_structs/0","title":"Cmp.Util.comparable_structs/0","type":"function"},{"doc":"","ref":"Cmp.Util.html#compare_terms/2","title":"Cmp.Util.compare_terms/2","type":"function"},{"doc":"Error raised when trying to compare values of different types.","ref":"Cmp.TypeError.html","title":"Cmp.TypeError","type":"exception"},{"doc":"Semantic comparison and sorting for Elixir.","ref":"readme.html","title":"Cmp","type":"extras"},{"doc":"The built-in comparison operators as well as functions like Enum.sort/2 or Enum.max/1 are based on Erlang's term ordering and suffer two issues, which require attention and might lead to unexpected behaviors or bugs: 1. Structural comparisons Built-ins use structural comparison over semantic comparison : iex&gt; ~D[2020-03-02] &gt; ~D[2019-06-06] false iex&gt; Enum . sort ( [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] ) [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] Semantic comparison is available but not straightforward: iex&gt; Date . compare ( ~D[2019-01-01] , ~D[2020-03-02] ) :lt iex&gt; Enum . sort ( [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] , Date ) [ ~D[2019-01-01] , ~D[2019-06-06] , ~D[2020-03-02] ] Cmp will do the right right out of the box: iex&gt; Cmp . gt? ( ~D[2020-03-02] , ~D[2019-06-06] ) true iex&gt; Cmp . sort ( [ ~D[2019-01-01] , ~D[2020-03-02] , ~D[2019-06-06] ] ) [ ~D[2019-01-01] , ~D[2019-06-06] , ~D[2020-03-02] ] 2. Weakly typed Built-ins will accept any set of operands and happily compare them: iex&gt; 2 &lt; &quot;1&quot; true iex&gt; 0 &lt; true true iex&gt; false &lt; nil true Cmp will only compare compatible elements or raise a Cmp.TypeError : iex&gt; Cmp . lte? ( 1 , 1.0 ) true iex&gt; Cmp . lte? ( 2 , &quot;1&quot; ) ** (Cmp.TypeError) Failed to compare incompatible types - left: 2, right: &quot;1&quot;","ref":"readme.html#why-cmp","title":"Cmp - Why Cmp?","type":"extras"},{"doc":"Cmp can be installed by adding cmp to your list of dependencies in mix.exs : def deps do [ { :cmp , &quot;~&gt; 0.1.0&quot; } ] end The documentation can be found at https://hexdocs.pm/cmp .","ref":"readme.html#installation","title":"Cmp - Installation","type":"extras"},{"doc":"Fast and well-optimized - the overhead should be quite small over built-in equivalents. See the benchmarks/ folder for more details. No need to require macros, plain functions Easily extensible through the Cmp.Comparable protocol Robust and well-tested (both unit and property-based)","ref":"readme.html#design-goals","title":"Cmp - Design goals","type":"extras"},{"doc":"Cmp is licensed under the MIT License .","ref":"readme.html#copyright-and-license","title":"Cmp - Copyright and License","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#dev","title":"Changelog - Dev","type":"extras"},{"doc":"Initial release","ref":"changelog.html#v0-1-0-2023-03-04","title":"Changelog - v0.1.0 (2023-03-04)","type":"extras"},{"doc":"MIT License Copyright (c) 2023 Sabiwara Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]